public inbox for linux-kernel@vger.kernel.org
 help / color / mirror / Atom feed
From: john stultz <johnstul@us.ibm.com>
To: David Miller <davem@davemloft.net>
Cc: tglx@linutronix.de, lkml <linux-kernel@vger.kernel.org>,
	Peter Keilty <peter.keilty@hp.com>
Subject: Re: sparc generic time / clockevents
Date: Fri, 23 Feb 2007 11:51:18 -0800	[thread overview]
Message-ID: <1172260279.6261.20.camel@localhost> (raw)
In-Reply-To: <20070223.015520.125893182.davem@davemloft.net>

On Fri, 2007-02-23 at 01:55 -0800, David Miller wrote:
> From: Thomas Gleixner <tglx@linutronix.de>
> Date: Fri, 23 Feb 2007 10:56:18 +0100
> 
> > On Fri, 2007-02-23 at 01:25 -0800, David Miller wrote:
> > > > Yes, all you need is to omit the CLOCK_EVT_FEAT_PERIODIC flag when you
> > > > register your device.
> > > 
> > > Thanks a lot Thomas.
> > > 
> > > I noticed while doing this work that the generic clock code is
> > > incompatible with the time interpolator, since both provide a
> > > do_{get,set}timeofday() implementation.  Just FYI...
> > 
> > John, can you have a look at this ?
> 
> Note we may not care :-)  All the interpolator does is provide
> a generic tick based gettimeofday implementation, and the generic
> clock code does essentially the same thing.
> 
> And the generic clock code must in fact provide the implementations
> in order to deal with dyntick issues.
> 
> The only other platform using the time interpolator is IA64,
> and after my dyntick implementation on sparc64 the one and
> only user will be IA64 :-)

Yea. I actually have some in-progress patches from Peter Keilty that
convert ia64 and sparc64 time_interpolators to clocksources, then
removes the time_interpolator code.

The ia64 conversion is more complicated due to the fsyscall asm, but I
think the sparc64 conversion (below) is pretty straight forward. I've
only built tested this, so I have no clue if it actually works. 

Any thoughts?

thanks
-john


From: Peter Keilty <peter.keilty@hp.com>

Initial sparc64 conversion to generic clocksource/timekeeping code.

Signed-off-by: Peter Keilty <peter.keilty@hp.com>
Signed-off-by: John Stultz <johnstul@us.ibm.com>


 Kconfig       |    2 +-
 defconfig     |    2 +-
 kernel/time.c |   34 +++++++++++++++++++++++-----------
 3 files changed, 25 insertions(+), 13 deletions(-)

linux-2.6.21-rc1_timeofday-arch-sparc64_C7.patch
============================================
diff --git a/arch/sparc64/Kconfig b/arch/sparc64/Kconfig
index f75a686..13a4547 100644
--- a/arch/sparc64/Kconfig
+++ b/arch/sparc64/Kconfig
@@ -34,7 +34,7 @@ config LOCKDEP_SUPPORT
 	bool
 	default y
 
-config TIME_INTERPOLATION
+config GENERIC_TIME
 	bool
 	default y
 
diff --git a/arch/sparc64/defconfig b/arch/sparc64/defconfig
index 0f44a6a..33e061a 100644
--- a/arch/sparc64/defconfig
+++ b/arch/sparc64/defconfig
@@ -9,7 +9,7 @@ CONFIG_64BIT=y
 CONFIG_MMU=y
 CONFIG_STACKTRACE_SUPPORT=y
 CONFIG_LOCKDEP_SUPPORT=y
-CONFIG_TIME_INTERPOLATION=y
+CONFIG_GENERIC_TIME=y
 CONFIG_ARCH_MAY_HAVE_PC_FDC=y
 # CONFIG_ARCH_HAS_ILOG2_U32 is not set
 # CONFIG_ARCH_HAS_ILOG2_U64 is not set
diff --git a/arch/sparc64/kernel/time.c b/arch/sparc64/kernel/time.c
index f84da4f..497b29b 100644
--- a/arch/sparc64/kernel/time.c
+++ b/arch/sparc64/kernel/time.c
@@ -31,6 +31,7 @@ #include <linux/percpu.h>
 #include <linux/profile.h>
 #include <linux/miscdevice.h>
 #include <linux/rtc.h>
+#include <linux/clocksource.h>
 
 #include <asm/oplib.h>
 #include <asm/mostek.h>
@@ -620,7 +621,7 @@ #endif
 	if (!mregs && !dregs) {
 		prom_printf("Something wrong, clock regs not mapped yet.\n");
 		prom_halt();
-	}		
+	}
 
 	if (mregs) {
 		spin_lock_irq(&mostek_lock);
@@ -820,7 +821,7 @@ #endif
 	}
 
 	set_system_time();
-	
+
 	local_irq_restore(flags);
 
 	return 0;
@@ -975,22 +976,33 @@ static struct notifier_block sparc64_cpu
 
 #endif /* CONFIG_CPU_FREQ */
 
-static struct time_interpolator sparc64_cpu_interpolator = {
-	.source		=	TIME_SOURCE_CPU,
-	.shift		=	16,
-	.mask		=	0xffffffffffffffffLL
+static cycle_t read_sparc64_cpuclock(void)
+{
+        return (cycle_t)get_cycles();
+}
+
+static struct clocksource clocksource_sparc64_cpuclock = {
+	.name           = "sparc64_cpuclock",
+	.rating         = 300,
+	.read           = read_sparc64_cpuclock,
+	.mask           = 0xffffffffffffffffLL,
+	.mult           = 0, /*to be caluclated*/
+	.shift          = 16,
+	.flags		= CLOCK_SOURCE_IS_CONTINUOUS,
 };
 
+
 /* The quotient formula is taken from the IA64 port. */
 #define SPARC64_NSEC_PER_CYC_SHIFT	10UL
 void __init time_init(void)
 {
 	unsigned long clock = sparc64_init_timers();
 
-	sparc64_cpu_interpolator.frequency = clock;
-	register_time_interpolator(&sparc64_cpu_interpolator);
+	clocksource_sparc64_cpuclock.mult = clocksource_hz2mult(clock,
+					clocksource_sparc64_cpuclock.shift);
+	clocksource_register(&clocksource_sparc64_cpuclock);
 
 	/* Now that the interpolator is registered, it is
 	 * safe to start the timer ticking.
 	 */
 	sparc64_start_timers();
@@ -1025,11 +1037,11 @@ #endif
 	unsigned long flags;
 	u8 tmp;
 
-	/* 
+	/*
 	 * Not having a register set can lead to trouble.
 	 * Also starfire doesn't have a tod clock.
 	 */
-	if (!mregs && !dregs) 
+	if (!mregs && !dregs)
 		return -1;
 
 	if (mregs) {
@@ -975,22 +976,33 @@ static struct notifier_block sparc64_cpu
 
-	/* Now that the interpolator is registered, it is
+	/* Now that the clocksource is registered, it is
 	 * safe to start the timer ticking.
 	 */



  reply	other threads:[~2007-02-23 19:51 UTC|newest]

Thread overview: 206+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2007-02-21  4:53 Linux 2.6.21-rc1 Linus Torvalds
2007-02-21 13:26 ` Faik Uygur
2007-02-21 18:41   ` Jiri Slaby
2007-02-21 18:51     ` Jiri Slaby
2007-02-21 19:19   ` Linus Torvalds
2007-02-21 13:32 ` Thomas Gleixner
2007-02-21 15:58   ` Kok, Auke
2007-02-21 19:24     ` Linus Torvalds
2007-02-21 19:45       ` Andrew Morton
2007-02-21 16:23 ` request_module: runaway loop modprobe net-pf-1 (is Re: Linux 2.6.21-rc1) YOSHIFUJI Hideaki / 吉藤英明
     [not found]   ` <87ps8372bf.fsf@duaron.myhome.or.jp>
2007-02-21 20:36     ` Greg KH
2007-02-21 21:16       ` OGAWA Hirofumi
2007-02-22  0:18         ` Greg KH
2007-02-22  9:57           ` Anders Larsen
2007-02-22 10:30             ` David Miller
     [not found]     ` <20070222.110440.47345562.yoshfuji@linux-ipv6.org>
     [not found]       ` <20070222.123417.79213825.yoshfuji@linux-ipv6.org>
2007-02-22  6:08         ` Greg KH
2007-02-21 16:24 ` Linux 2.6.21-rc1 Daniel Walker
2007-02-21 17:07   ` Thomas Gleixner
2007-02-21 17:19     ` Daniel Walker
2007-02-21 17:41       ` Thomas Gleixner
2007-02-21 17:38         ` Daniel Walker
2007-02-21 18:18           ` Thomas Gleixner
2007-02-21 18:23             ` Daniel Walker
2007-02-21 19:23               ` Thomas Gleixner
2007-02-21 19:24                 ` Daniel Walker
2007-02-21 20:00                 ` Daniel Walker
2007-02-21 20:18                   ` Linus Torvalds
2007-02-21 20:43                   ` Thomas Gleixner
2007-02-21 20:49                     ` Daniel Walker
2007-02-21 21:06                       ` Linus Torvalds
2007-02-21 21:21                         ` Thomas Gleixner
2007-02-21 21:23                         ` Daniel Walker
2007-02-21 22:05                         ` Linux 2.6.21-rc1 [git bisect] Pete Harlan
2007-02-23 10:08         ` Linux 2.6.21-rc1 Andrew Morton
2007-02-23 11:35           ` Ingo Molnar
2007-02-23 11:39             ` Ingo Molnar
2007-02-23 11:47             ` Thomas Gleixner
2007-02-21 18:34 ` Andreas Schwab
2007-02-21 18:40   ` Dave Jones
2007-02-21 23:04 ` NO_HZ: timer interrupt stuck [Re: Linux 2.6.21-rc1] Luca Tettamanti
2007-02-21 23:17   ` Thomas Gleixner
2007-02-21 23:19     ` Luca Tettamanti
2007-02-22 12:36     ` Jan Engelhardt
2007-02-22 13:25       ` Arjan van de Ven
2007-02-22 14:10         ` Pierre Ossman
2007-02-22 14:20           ` Arjan van de Ven
2007-02-22 14:51             ` Pierre Ossman
2007-02-22 15:13               ` Pierre Ossman
2007-02-22 16:00                 ` Thomas Gleixner
2007-02-22 16:27                   ` Pierre Ossman
2007-02-22 16:42                     ` Arjan van de Ven
2007-02-22 21:07                       ` Pierre Ossman
2007-02-22 21:25                         ` Andreas Mohr
2007-02-22 22:21                         ` Arjan van de Ven
2007-02-23  6:55                           ` Pierre Ossman
2007-02-22 19:58                   ` Pallipadi, Venkatesh
2007-02-22 15:51       ` Thomas Gleixner
2007-02-22 17:26         ` NO_HZ: timer interrupt stuck David Miller
2007-02-22 17:39           ` Thomas Gleixner
2007-02-23  9:25             ` David Miller
2007-02-23  9:56               ` sparc generic time / clockevents [ was Re: NO_HZ: timer interrupt stuck ] Thomas Gleixner
2007-02-23  9:55                 ` sparc generic time / clockevents David Miller
2007-02-23 19:51                   ` john stultz [this message]
2007-02-23 22:15                     ` Peter Keilty
2007-02-24  0:34                     ` David Miller
2007-02-24  0:53                       ` john stultz
2007-02-24  5:52                         ` David Miller
2007-02-25  5:13                       ` generic one-shot bug (was Re: sparc generic time / clockevents) David Miller
2007-02-25  8:34                         ` Thomas Gleixner
2007-02-23 15:50           ` NO_HZ: timer interrupt stuck Andi Kleen
2007-02-23 15:48     ` NO_HZ: timer interrupt stuck [Re: Linux 2.6.21-rc1] Andi Kleen
2007-02-23  5:25 ` Linux 2.6.21-rc1 -- suspend Pavel Machek
2007-02-25 17:52 ` 2.6.21-rc1: known regressions (part 1) Adrian Bunk
2007-02-27  8:39   ` regression: forcedeth.c hang Ingo Molnar
2007-02-27  9:01     ` Ingo Molnar
2007-02-27  9:38       ` Ingo Molnar
2007-02-27 11:25         ` Ingo Molnar
2007-02-27 15:42           ` Linus Torvalds
2007-02-28  7:36             ` Ingo Molnar
2007-02-28 18:16   ` 2.6.21-rc1: known regressions (part 1) Karasyov, Konstantin A
2007-02-25 17:55 ` 2.6.21-rc1: known regressions (part 2) Adrian Bunk
2007-02-27 10:02   ` Jens Axboe
2007-02-27 10:21     ` Pavel Machek
2007-02-27 10:30       ` Jens Axboe
2007-02-27 10:34         ` Ingo Molnar
2007-02-27 10:59           ` Jens Axboe
2007-02-27 11:15             ` Jens Axboe
2007-02-27 13:09               ` Jens Axboe
2007-03-01  9:34               ` Ingo Molnar
2007-03-01 10:41                 ` Ingo Molnar
2007-03-01 14:52                   ` Ingo Molnar
2007-03-01 16:12                     ` Rafael J. Wysocki
2007-03-02  0:26                     ` Linus Torvalds
2007-03-02  0:41                       ` Linus Torvalds
2007-03-02  7:14                       ` Ingo Molnar
2007-03-02  7:21                       ` Ingo Molnar
2007-03-02  8:04                         ` Ingo Molnar
2007-03-02 10:20                           ` Ingo Molnar
2007-03-02 10:22                             ` [patch] KVM: T60 resume fix Ingo Molnar
2007-03-02 11:39                               ` Michael S. Tsirkin
2007-03-03  8:22                                 ` Avi Kivity
2007-03-03  8:21                               ` Avi Kivity
2007-03-03 11:57                                 ` Andrew Morton
2007-03-03 12:07                                   ` Junio C Hamano
2007-03-05  8:22                                 ` Ingo Molnar
2007-03-05  8:50                                   ` Avi Kivity
2007-03-05  8:44                                     ` Ingo Molnar
2007-03-05  8:57                                       ` Ingo Molnar
2007-03-05  9:27                                         ` Avi Kivity
2007-03-05 10:05                                           ` Ingo Molnar
2007-03-05 10:33                                             ` Avi Kivity
2007-03-05 10:33                                               ` Ingo Molnar
2007-03-05 10:40                                               ` Michael S. Tsirkin
2007-03-05 12:54                                         ` Michael S. Tsirkin
2007-03-05 12:50                                           ` Ingo Molnar
2007-03-05 13:26                                             ` Michael S. Tsirkin
2007-03-05 13:32                                               ` Ingo Molnar
2007-03-05 10:23                               ` Michael S. Tsirkin
2007-03-05 10:29                                 ` Ingo Molnar
2007-03-05 15:44                           ` 2.6.21-rc1: known regressions (part 2) Michael S. Tsirkin
2007-03-05 16:14                           ` Michael S. Tsirkin
2007-03-05 16:41                             ` Ingo Molnar
2007-03-05 18:16                             ` Jens Axboe
2007-03-01 23:36                   ` Linus Torvalds
2007-03-02 10:07                 ` Pavel Machek
2007-03-05  8:42                   ` Michael S. Tsirkin
2007-03-05 10:11                     ` SATA resume slowness, e1000 MSI warning Ingo Molnar
2007-03-06  5:30                       ` Jeff Garzik
2007-03-06  6:35                         ` Kok, Auke
2007-03-06  9:04                           ` Ingo Molnar
2007-03-06 15:34                             ` Kok, Auke
2007-03-07  4:15                               ` Eric W. Biederman
2007-03-07 16:31                                 ` Kok, Auke
2007-03-07 16:45                                   ` Kok, Auke
2007-03-07 19:28                                     ` Eric W. Biederman
2007-03-08  2:53                                       ` Andrew Morton
2007-03-08  6:58                                         ` Eric W. Biederman
2007-03-08  9:55                                           ` Jeff Garzik
2007-03-08 17:27                                             ` Eric W. Biederman
2007-03-08 19:58                                               ` [PATCH 0/2] Repair pci_restore_state when used with device resets Eric W. Biederman
2007-03-08 20:04                                                 ` [PATCH 1/2] msi: Safer state caching Eric W. Biederman
2007-03-08 20:06                                                   ` [PATCH 2/2] pci: Repair pci_save/restore_state so we can restore one save many times Eric W. Biederman
2007-03-10  7:50                                                     ` patch pci-repair-pci_save-restore_state-so-we-can-restore-one-save-many-times.patch added to gregkh-2.6 tree gregkh
2007-03-12 22:46                                                     ` [PATCH 2/2] pci: Repair pci_save/restore_state so we can restore one save many times Kok, Auke
2007-03-10  7:49                                                   ` patch msi-safer-state-caching.patch added to gregkh-2.6 tree gregkh
2007-03-08 20:08                                                 ` [PATCH 0/2] Repair pci_restore_state when used with device resets Ingo Molnar
2007-03-08 20:26                                                   ` Eric W. Biederman
2007-03-08 10:23                                           ` SATA resume slowness, e1000 MSI warning Michael S. Tsirkin
2007-03-11 11:11                                             ` Eric W. Biederman
2007-03-11 11:24                                               ` Michael S. Tsirkin
2007-03-11 17:37                                                 ` Eric W. Biederman
2007-03-11 18:03                                                   ` Michael S. Tsirkin
2007-03-11 18:27                                                     ` Eric W. Biederman
2007-03-11 18:37                                                       ` Michael S. Tsirkin
2007-03-11 19:50                                                         ` Eric W. Biederman
2007-03-12  4:35                                                           ` Michael S. Tsirkin
2007-04-16 19:56                                                 ` Michael S. Tsirkin
2007-03-09 23:06                                       ` Kok, Auke
2007-03-10  3:41                                         ` Eric W. Biederman
2007-03-06  9:06                       ` Ingo Molnar
2007-03-06 16:26                       ` Thomas Gleixner
2007-03-06 16:52                         ` Linus Torvalds
2007-03-06 17:09                           ` Kok, Auke
2007-03-09  6:44                     ` [linux-pm] 2.6.21-rc1: known regressions (part 2) Pavel Machek
2007-03-05 15:34                 ` Michael S. Tsirkin
2007-02-27 22:09     ` Adrian Bunk
2007-02-28  7:41       ` Jens Axboe
2007-02-25 18:02 ` 2.6.21-rc1: known regressions (part 3) Adrian Bunk
2007-02-25 20:59   ` Greg KH
2007-02-26 22:01 ` 2.6.21-rc1: known regressions (v2) (part 1) Adrian Bunk
2007-02-27 13:00   ` Meelis Roos
2007-02-27 14:16     ` Alan
2007-02-28 21:13   ` Michael S. Tsirkin
2007-02-28 21:27     ` Thomas Gleixner
2007-02-28 21:40       ` Michael S. Tsirkin
2007-03-01  3:45     ` Jeff Chua
2007-03-02 12:26       ` [linux-pm] " Pavel Machek
2007-03-03 11:17         ` Jens Axboe
2007-03-05  0:04       ` Adrian Bunk
2007-03-06  1:32         ` Jeff Chua
2007-03-06 12:03           ` Jeff Chua
2007-03-06 12:08             ` Michael S. Tsirkin
2007-03-06 12:12               ` Jeff Chua
2007-03-19 15:32                 ` Pavel Machek
2007-03-19 21:23                   ` Rafael J. Wysocki
2007-02-26 22:05 ` 2.6.21-rc1: known regressions (v2) (part 2) Adrian Bunk
2007-02-27  8:21   ` Thomas Gleixner
2007-02-27  8:33     ` Michal Piotrowski
2007-02-27  8:33       ` Ingo Molnar
2007-02-27  8:54         ` Mike Galbraith
2007-02-27 23:07           ` Con Kolivas
2007-02-28  4:21             ` Mike Galbraith
2007-02-28 22:01               ` Con Kolivas
2007-03-01  0:02                 ` Mike Galbraith
2007-03-01  8:46                   ` Ingo Molnar
2007-03-01 11:13                     ` Con Kolivas
2007-03-01 11:33                       ` Thomas Gleixner
2007-03-01 12:05                         ` Con Kolivas
2007-03-01 12:20                           ` Thomas Gleixner
2007-03-01 13:30                           ` Ingo Molnar
2007-03-01 21:51                             ` Con Kolivas
2007-03-01 22:33                               ` [PATCH] sched: remove SMT nice Con Kolivas
     [not found]                                 ` <20070301173002.456f9534.akpm@linux-foundation.org>
2007-03-02  1:25                                   ` Con Kolivas
2007-02-27  8:35       ` 2.6.21-rc1: known regressions (v2) (part 2) Michal Piotrowski
     [not found] ` <200702271525.48645.ismail@pardus.org.tr>
     [not found]   ` <b637ec0b0702270614i25b6be9fmfb4b12ddd789a467@mail.gmail.com>
2007-02-27 18:44     ` 2.6.21-rc1: known regressions (v2) (part 1) S.Çağlar Onur
2007-02-27 19:08       ` S.Çağlar Onur

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=1172260279.6261.20.camel@localhost \
    --to=johnstul@us.ibm.com \
    --cc=davem@davemloft.net \
    --cc=linux-kernel@vger.kernel.org \
    --cc=peter.keilty@hp.com \
    --cc=tglx@linutronix.de \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox